/*===========================================================================*/
size_t nlmsg_datalen(const struct nlmsghdr *nlh)
{
return nlh->nlmsg_len -sizeof(struct nlmsghdr) -sizeof(struct genlmsghdr);
}
/*---------------------------------------------------------------------------*/
void *nlmsg_data(const struct nlmsghdr *nlh)
{
return ((NLMSG_DATA(nlh) + sizeof(struct genlmsghdr)));
//return (u8*)nlh + NLMSG_HDRLEN + sizeof(struct genlmsghdr);
}
/*===========================================================================*/
struct nlattr *nla_next(const struct nlattr *nla, size_t *remaining)
{
size_t totlen = NLA_ALIGN(nla->nla_len);

*remaining -= totlen;
return (struct nlattr*) ((u8*)nla +totlen);
}
/*---------------------------------------------------------------------------*/
size_t nla_ok(const struct nlattr *nla, size_t remaining)
{
return remaining >= (int) sizeof(*nla) && nla->nla_len >= sizeof(*nla) && nla->nla_len <= remaining;
}
/*---------------------------------------------------------------------------*/
int nla_datalen(const struct nlattr *nla)
{
return nla->nla_len -NLA_HDRLEN;
}
/*---------------------------------------------------------------------------*/
void *nla_data(const struct nlattr *nla)
{
return (u8*)nla +NLA_HDRLEN;
}
/*===========================================================================*/
static void get_supported_freqs(ifinfo_t *iflistakt, struct nlattr* nla)
{
struct nlattr *pos = NULL;
struct nlattr *index;
size_t remlen = 0;

index = (struct nlattr*)  nla_data(nla);
for (pos = (struct nlattr*)  nla_data(index), remlen = nla_datalen(index); nla_ok(pos, remlen); pos = nla_next(pos, &(remlen)))
	{
	if(pos->nla_type == NL80211_BAND_ATTR_FREQS)
		{
		iflistakt->frequency[iflistakt->cf] = *((u32*)nla_data(pos));
		iflistakt->channel[iflistakt->cf] = frequency_to_channel(iflistakt->frequency[iflistakt->cf]);
		}
	else if(pos->nla_type == NL80211_FREQUENCY_ATTR_MAX_TX_POWER) iflistakt->txpwr[iflistakt->cf] = *((u32*)nla_data(pos));
	else if(pos->nla_type == NL80211_FREQUENCY_ATTR_DISABLED) iflistakt->frequency[iflistakt->cf] = 0;
	}
if(iflistakt->frequency[iflistakt->cf] == 0) return;
if(iflistakt->cf > FREQMAX -1) return;
iflistakt->cf += 1;
return;
}

/*---------------------------------------------------------------------------*/
static void get_supported_bands(ifinfo_t *iflistakt, struct nlattr* nla)
{
struct nlattr *pos = NULL;
struct nlattr *index;
size_t remlen = 0;

index = (struct nlattr*)  nla_data(nla);
for (pos = (struct nlattr*) nla_data(index), remlen = nla_datalen(index); nla_ok(pos, remlen); pos = nla_next(pos, &(remlen)))
	{
	if(pos->nla_type == NL80211_BAND_ATTR_FREQS) get_supported_freqs(iflistakt, pos); 
	}
return;
}
/*---------------------------------------------------------------------------*/
static void get_supported_ifftypes(ifinfo_t *iflistakt, struct nlattr* nla)
{
struct nlattr *pos = NULL;
size_t remlen = 0;

for (pos = (struct nlattr*) nla_data(nla), remlen = nla_datalen(nla); nla_ok(pos, remlen); pos = nla_next(pos, &(remlen)))
	{
	if(pos->nla_type == NL80211_IFTYPE_MONITOR) iflistakt->iftype |= IFTYPEMON;
	}
return;
}
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
static bool set_nl80211_frequency()
{
struct nlmsghdr *nlh = NULL;
struct nlattr32 *nla32 = NULL;
struct nlmsgerr *nle = NULL;
ssize_t nlrxlen = 0;
u8 nlmd[] =
{
0x18, 0x00, 0x00, 0x00, /* nlmsg_len */
0x22, 0x00, /* nlmsg_type */
0x05, 0x00, /*nlmsg_flags */
0x00, 0x00, 0x00, 0x00, /* nlmsg_seq */
0x00, 0x00, 0x00, 0x00, /* nlmsg_pid */
0x02, /* nlg cmd */
0x00, /* nlg version */
0x00, 0x00, /* nlg reserved */
0x08, 0x00,
0x03, 0x00,
0x06, 0x00, 0x00, 0x00,
0x08, 0x00,
0x26, 0x00,
0x3c, 0x14, 0x00, 0x00,
0x08, 0x00,
0x9f, 0x00,
0x00, 0x00, 0x00, 0x00,
0x08, 0x00,
0x27, 0x00,
0x00, 0x00, 0x00, 0x00,
0x08, 0x00,
0xa0, 0x00,
0x3c, 0x14, 0x00, 0x00
};
#define NLMD_SIZE sizeof(nlmd)

if(scanlist[++scanlistcount] == 0) scanlistcount = 0;
nlh = (struct nlmsghdr *)nlmd;
nlh->nlmsg_len = NLMD_SIZE;
nlh->nlmsg_type = nlfamily;
nlh->nlmsg_seq = ++nlseqnr;
nla32 = (struct nlattr32*)(nlmd +24);
nla32->nla_val32 = ifakt->ifindex;
nlh->nlmsg_pid = hcxpid;
nla32 = (struct nlattr32*)(nlmd +32);
nla32->nla_val32 = scanlist[scanlistcount];
nla32 = (struct nlattr32*)(nlmd +56);
nla32->nla_val32 = scanlist[scanlistcount];
if(send(fd_socket_nl, nlmd, NLMD_SIZE, MSG_NOSIGNAL) != NLMD_SIZE) return 1;
while(1)
	{
	nlrxlen = recv(fd_socket_nl, nlrxb, NLMSG_RXB_SIZE, 0);
	if(nlrxlen <= 0) break;
	for(nlh = (struct nlmsghdr*)nlrxb; NLMSG_OK(nlh, nlrxlen); nlh = NLMSG_NEXT(nlh, nlrxlen))
		{
		switch(nlh->nlmsg_type)
			{
			case NLMSG_DONE:
			return true;
			break;

			case NLMSG_ERROR:
			nle = (struct nlmsgerr*)(nlrxb + sizeof(struct nlmsghdr));
			if(nle->error == 0) return true;
			return false;
			}
		}
	}
return false;
}
/*---------------------------------------------------------------------------*/
static size_t request_nl80211_interfacefeature()
{
struct nlmsghdr *nlh = NULL;
struct genlmsghdr *nlg = NULL;
struct nlattr *nla = NULL;
struct nlmsgerr *nle = NULL;
ssize_t nlrxlen = 0;
size_t nlaremlen = 0;
size_t c;
u8 nlmd[] =
{
0x18, 0x00, 0x00, 0x00, /* nlmsg_len */
0x22, 0x00, /* nlmsg_type */
0x05, 0x03, /*nlmsg_flags */
0x00, 0x00, 0x00, 0x00, /* nlmsg_seq */
0x00, 0x00, 0x00, 0x00, /* nlmsg_pid */
0x01, /* nlg cmd */
0x00, /* nlg version */
0x00, 0x00, /* nlg reserved */
0x04, 0x00, /* nla_len */
0xae, 0x00 /* nla_ype */
};
#define NLMD_SIZE sizeof(nlmd)

nlh = (struct nlmsghdr *)nlmd;
nlh->nlmsg_len = NLMD_SIZE;
nlh->nlmsg_type = nlfamily;
nlh->nlmsg_seq = ++nlseqnr;
nlh->nlmsg_pid = hcxpid;
if(send(fd_socket_nl, nlmd, NLMD_SIZE, MSG_NOSIGNAL) != NLMD_SIZE) return 1;
while(1)
	{
	nlrxlen = recv(fd_socket_nl, nlrxb, NLMSG_RXB_SIZE, 0);
	if(nlrxlen <= 0) break;
	c = IFINFOLIST_MAX -1;
	nlaremlen = 0;
	for(nlh = (struct nlmsghdr*)nlrxb; NLMSG_OK(nlh, nlrxlen); nlh = NLMSG_NEXT(nlh, nlrxlen))
		{
		switch(nlh->nlmsg_type)
			{
			case NLMSG_DONE:
			return nlaremlen;
			break;

			case NLMSG_ERROR:
			nle = (struct nlmsgerr*)(nlrxb + sizeof(struct nlmsghdr));
			if(nle->error == 0) return nlaremlen;
			return 1;

			default:
			if(nlh->nlmsg_type != nlfamily) continue;
			nlg = (struct genlmsghdr*)(nlh +1);
			if(nlg->cmd != NL80211_CMD_NEW_WIPHY) continue;
			for(nla = (struct nlattr*)nlmsg_data(nlh), nlaremlen = nlmsg_datalen(nlh); nla_ok(nla, nlaremlen); nla = nla_next(nla, &(nlaremlen)))
				{
				if(nla->nla_type == NL80211_ATTR_WIPHY)
					{
					for(c = 0; c < ifc; c++)
						{
						if((iflist +c)->phyindex == *((u32*)nla_data(nla))) break;
						}
					}
				else if(nla->nla_type == NL80211_ATTR_SUPPORTED_IFTYPES) get_supported_ifftypes((iflist +c), nla);
				else if(nla->nla_type == NL80211_ATTR_WIPHY_BANDS) get_supported_bands((iflist +c), nla);
				}
			}
		}
	}
return nlaremlen;
}
/*---------------------------------------------------------------------------*/
static bool request_nl80211_interfaces()
{
struct nlmsghdr *nlh = NULL;
struct genlmsghdr *nlg = NULL;
struct nlattr *nla = NULL, *nlas = NULL;
struct nlmsgerr *nle = NULL;
ssize_t nlrxlen = 0;
size_t nlaremlen = 0;
u8 nlmd[] =
{
0x14, 0x00, 0x00, 0x00, /* nlmsg_len */
0x22, 0x00, /* nlmsg_type */
0x05, 0x03, /*nlmsg_flags */
0x00, 0x00, 0x00, 0x00, /* nlmsg_seq */
0x00, 0x00, 0x00, 0x00, /* nlmsg_pid */
0x05, /* nlg cmd */
0x00, /* nlg version */
0x00, 0x00 /* nlg reserved */
};
#define NLMD_SIZE sizeof(nlmd)

nlh = (struct nlmsghdr *)nlmd;
nlh->nlmsg_len = NLMD_SIZE;
nlh->nlmsg_type = nlfamily;
nlh->nlmsg_seq = ++nlseqnr;
nlh->nlmsg_pid = hcxpid;
if(send(fd_socket_nl, nlmd, NLMD_SIZE, MSG_NOSIGNAL) != NLMD_SIZE) return false;
while(1)
	{
	nlrxlen = recv(fd_socket_nl, nlrxb, NLMSG_RXB_SIZE, 0);
	if(nlrxlen <= 0) break;
	nlaremlen = 0;
	for(nlh = (struct nlmsghdr*)nlrxb; NLMSG_OK(nlh, nlrxlen); nlh = NLMSG_NEXT(nlh, nlrxlen))
		{
		switch(nlh->nlmsg_type)
			{
			case NLMSG_DONE:
			return true;
			break;

			case NLMSG_ERROR:
			nle = (struct nlmsgerr*)(nlrxb + sizeof(struct nlmsghdr));
			if(nle->error == 0) return true;
			return false;

			default:
			if(ifc > IFINFOLIST_MAX) continue;
			if(nlh->nlmsg_type != nlfamily) continue;
			nlg = (struct genlmsghdr*)(nlh +1);
			if(nlg->cmd != NL80211_CMD_NEW_INTERFACE) continue;
			nlas = (struct nlattr*)nlmsg_data(nlh);
			for(nla = nlas, nlaremlen = nlmsg_datalen(nlh); nla_ok(nla, nlaremlen); nla = nla_next(nla, &(nlaremlen)))
				{
				if(nla->nla_type == NL80211_ATTR_IFINDEX) (iflist +ifc)->ifindex = *((u32*)nla_data(nla));
				else if(nla->nla_type == NL80211_ATTR_WIPHY)
					{
					(iflist +ifc)->phyindex = *((u32*)nla_data(nla));
					}
				else if(nla->nla_type == NL80211_ATTR_IFNAME)
					{
					if((nla_datalen(nla) > 0) && (nla_datalen(nla) <= IFNAMSIZ))
						{
						memset((iflist +ifc)->ifname, 0, IFNAMSIZ);
						memcpy((iflist +ifc)->ifname, nla_data(nla), nla_datalen(nla));
						(iflist +ifc)->iftype = IFTYPENL;
						}
					}
				else if(nla->nla_type == NL80211_ATTR_MAC) memcpy((iflist +ifc)->vimac, nla_data(nla), IFHWADDRLEN);
				}
			if((iflist +ifc)->ifindex != 0) ifc++;
			else memset(iflist +ifc, 0 , IFINFOLIST_SIZE);
			}
		}
	}
return false;
}
/*---------------------------------------------------------------------------*/
static bool request_nl80211_protocolfeatures()
{
struct nlmsghdr *nlh = NULL;
struct genlmsghdr *nlg = NULL;
struct nlattr *nla = NULL, *nlas = NULL;
struct nlmsgerr *nle = NULL;
ssize_t nlrxlen = 0;
size_t nlaremlen = 0;
bool retv = false;
u8 nlmd[] =
{
0x14, 0x00, 0x00, 0x00, /* nlmsg_len */
0x22, 0x00, /* nlmsg_type */
0x05, 0x00, /*nlmsg_flags */
0x00, 0x00, 0x00, 0x00, /* nlmsg_seq */
0x00, 0x00, 0x00, 0x00, /* nlmsg_pid */
0x5f, /* nlg cmd */
0x00, /* nlg version */
0x00, 0x00 /* nlg reserved */
};
#define NLMD_SIZE sizeof(nlmd)

nlh = (struct nlmsghdr *)nlmd;
nlh->nlmsg_len = NLMD_SIZE;
nlh->nlmsg_type = nlfamily;
nlh->nlmsg_seq = ++nlseqnr;
nlh->nlmsg_pid = hcxpid;
if(send(fd_socket_nl, nlmd, NLMD_SIZE, MSG_NOSIGNAL) != NLMD_SIZE) return retv;
while(1)
	{
	nlrxlen = recv(fd_socket_nl, nlrxb, NLMSG_RXB_SIZE, 0);
	if(nlrxlen <= 0) break;
	nlaremlen = 0;
	for(nlh = (struct nlmsghdr*)nlrxb; NLMSG_OK(nlh, nlrxlen); nlh = NLMSG_NEXT(nlh, nlrxlen))
		{
		switch(nlh->nlmsg_type)
			{
			case NLMSG_DONE:
			return retv;
			break;

			case NLMSG_ERROR:
			nle = (struct nlmsgerr*)(nlrxb + sizeof(struct nlmsghdr));
			if(nle->error == 0) return retv;
			return false;

			default:
			if(nlh->nlmsg_type != nlfamily) continue;
			nlg = (struct genlmsghdr*)(nlh +1);
			if(nlg->cmd != NL80211_CMD_GET_PROTOCOL_FEATURES) continue;
			nlas = (struct nlattr*)nlmsg_data(nlh);
			for(nla = nlas, nlaremlen = nlmsg_datalen(nlh); nla_ok(nla, nlaremlen); nla = nla_next(nla, &(nlaremlen)))
				{
				if(nla->nla_type == NL80211_ATTR_PROTOCOL_FEATURES)
					{
					if(*((u32*)nla_data(nla)) == 1) retv = true;
					}
				}
			}
		}
	}
return retv;
}
/*---------------------------------------------------------------------------*/
static bool request_nl80211_family()
{
struct nlmsghdr *nlh = NULL;
struct genlmsghdr *nlg = NULL;
struct nlattr *nla = NULL, *nlas = NULL;
struct nlmsgerr *nle = NULL;
ssize_t nlrxlen = 0;
size_t nlaremlen = 0;
u8 nlmd[] =
{
0x20, 0x00, 0x00, 0x00, /* nlmsg_len */
0x10, 0x00, /* nlmsg_type */
0x05, 0x01, /*nlmsg_flags */
0x00, 0x00, 0x00, 0x00, /* nlmsg_seq */
0x00, 0x00, 0x00, 0x00, /* nlmsg_pid */
0x03, /* nlg cmd */
0x00, /* nlg version */
0x00, 0x00, /* nlg reserved */
0x0c, 0x00, /* nla_len */
0x02, 0x00, /* nla_type */
0x6e, 0x6c, 0x38, 0x30, 0x32, 0x31, 0x31, /* NL80211_GENL_NAME */
0x00 /* padding */
};
#define NLMD_SIZE sizeof(nlmd)

nlh = (struct nlmsghdr *)nlmd;
nlh->nlmsg_len = NLMD_SIZE;
nlh->nlmsg_seq = ++nlseqnr;
nlh->nlmsg_pid = hcxpid;
if(send(fd_socket_nl, nlmd, NLMD_SIZE, MSG_NOSIGNAL) != NLMD_SIZE) return false;
while(1)
	{
	nlrxlen = recv(fd_socket_nl, nlrxb, NLMSG_RXB_SIZE, 0);
	if(nlrxlen <= 0) break;
	for(nlh = (struct nlmsghdr*)nlrxb; NLMSG_OK(nlh, nlrxlen); nlh = NLMSG_NEXT(nlh, nlrxlen))
		{
		switch(nlh->nlmsg_type)
			{
			case NLMSG_DONE:
			return true;
			break;

			case NLMSG_ERROR:
			nle = (struct nlmsgerr*)(nlrxb + sizeof(struct nlmsghdr));
			if(nle->error == 0) return true;
			return false;

			default:
			if(nlh->nlmsg_type != GENL_ID_CTRL) continue;
			nlg = (struct genlmsghdr*)(nlh +1);
			if(nlg->cmd != CTRL_CMD_NEWFAMILY) continue;
			nlas = (struct nlattr*)nlmsg_data(nlh);
			for(nla = nlas, nlaremlen = nlmsg_datalen(nlh); nla_ok(nla, nlaremlen); nla = nla_next(nla, &(nlaremlen)))
				{
				if(nla->nla_type == CTRL_ATTR_FAMILY_ID) nlfamily = *((u32*)nla_data(nla));
				}
			}
		}
	}
return false;
}
/*===========================================================================*/
static bool set_nl80211_monitor_mode()
{
struct nlmsghdr *nlh = NULL;
struct nlattr32 *nla32 = NULL;
struct nlmsgerr *nle = NULL;
ssize_t nlrxlen = 0;
u8 nlmd[] =
{
0x20, 0x00, 0x00, 0x00, /* nlmsg_len */
0x22, 0x00, /* nlmsg_type */
0x05, 0x00, /*nlmsg_flags */
0x00, 0x00, 0x00, 0x00, /* nlmsg_seq */
0x00, 0x00, 0x00, 0x00, /* nlmsg_pid */
0x06, /* nlg cmd */
0x00, /* nlg version */
0x00, 0x00, /* nlg reserved */
0x08, 0x00, /* nla_len */
0x03, 0x00, /* nla_type ifindex */
0x00, 0x00, 0x00, 0x00, /* nla attribute */
0x08, 0x00, /* nla_len */
0x05, 0x00, /* nla_type monitor mode*/
0x06, 0x00, 0x00, 0x00, /* nla attribute */
0x08, 0x00, /* nla_len */
0x17, 0x00,/* nla_type */
0x04, 0x00, 0x06, 0x00 /* nla attribute active*/
};
#define NLMD_SIZE sizeof(nlmd)

nlh = (struct nlmsghdr*)nlmd;
nlh->nlmsg_len = NLMD_SIZE;
nlh->nlmsg_type = nlfamily;
nlh->nlmsg_seq = ++nlseqnr;
nlh->nlmsg_pid = hcxpid;
nla32 = (struct nlattr32*)(nlmd +24);
nla32->nla_val32 = ifakt->ifindex;

nlh->nlmsg_len = NLMD_SIZE -8;
if(send(fd_socket_nl, nlmd, NLMD_SIZE -8, MSG_NOSIGNAL) != NLMD_SIZE -8) return false;
while(1)
	{
	nlrxlen = recv(fd_socket_nl, nlrxb, NLMSG_RXB_SIZE, 0);
	if(nlrxlen <= 0) break;
	for(nlh = (struct nlmsghdr*)nlrxb; NLMSG_OK(nlh, nlrxlen); nlh = NLMSG_NEXT(nlh, nlrxlen))
		{
		switch(nlh->nlmsg_type)
			{
			case NLMSG_DONE:
			return nlfamily;
			break;

			case NLMSG_ERROR:
			nle = (struct nlmsgerr*)(nlrxb + sizeof(struct nlmsghdr));
			if(nle->error == 0) return nlfamily;
			return false;
			}
		}
	}
return true;
}
/*===========================================================================*/
static bool set_rt_interface_down(u32 condition)
{
struct nlmsghdr *nlh = NULL;
struct ifinfomsg *ifi = NULL;
struct nlmsgerr *nle = NULL;
ssize_t nlrxlen = 0;
u8 rtmd[] =
{
0x20, 0x00, 0x00, 0x00, /* nlmsg_len */
0x10, 0x00, /* nlmsg_type */
0x05, 0x00, /*nlmsg_flags */
0x00, 0x00, 0x00, 0x00, /* nlmsg_seq */
0x00, 0x00, 0x00, 0x00, /* nlmsg_pid */
0x00, /* rta_len */
0x00, /* rta_type */
0x00, /* ifi_family */
0x00, /* ifi_type */
0x00, 0x00, 0x00, 0x00, /* ifi_index */
0x00, 0x00, 0x00, 0x00, /* ifi_flags */
0x01, 0x00, 0x00, 0x00 /* ifi_change */
};
#define RTMD_SIZE sizeof(rtmd)

nlh = (struct nlmsghdr *)rtmd;
nlh->nlmsg_len = RTMD_SIZE;
nlh->nlmsg_seq = ++nlseqnr;
nlh->nlmsg_pid = hcxpid;
ifi = (struct ifinfomsg*)(rtmd +16);
ifi->ifi_index = ifakt->ifindex;
ifi->ifi_flags = condition;
if(send(fd_socket_rt, rtmd, RTMD_SIZE, MSG_NOSIGNAL) != RTMD_SIZE) return false;
while(1)
	{
	nlrxlen = recv(fd_socket_rt, nlrxb, NLMSG_RXB_SIZE, 0);
	if(nlrxlen <= 0) break;
	for(nlh = (struct nlmsghdr*)nlrxb; NLMSG_OK(nlh, nlrxlen); nlh = NLMSG_NEXT(nlh, nlrxlen))
		{
		switch(nlh->nlmsg_type)
			{
			case NLMSG_DONE:
			return true;
			break;

			case NLMSG_ERROR:
			nle = (struct nlmsgerr*)(nlrxb + sizeof(struct nlmsghdr));
			if(nle->error == 0) return true;
			return false;
			}
		}
	}
return false;
}
/*===========================================================================*/
