/*===========================================================================*/
static u16 addoption(u8 *posopt, u16 optioncode, u16 optionlen, char *option)
{
u16 padding;
option_header_t *optionhdr;

optionhdr = (option_header_t*)posopt;
optionhdr->option_code = optioncode;
optionhdr->option_length = optionlen;
padding = (4 -(optionlen %4)) %4;
memset(optionhdr->option_data, 0, optionlen +padding); 
memcpy(optionhdr->option_data, option, optionlen);
return optionlen + padding +4;
}
/*---------------------------------------------------------------------------*/
static u16 addcustomoptionheader(u8 *pospt)
{
u16 colen;
option_header_t *optionhdr;

optionhdr = (option_header_t*)pospt;
optionhdr->option_code = SHB_CUSTOM_OPT;
colen = OH_SIZE;
memcpy(pospt +colen, &hcxmagic, 4);
colen += 4;
memcpy(pospt +colen, &hcxmagic, 32);
colen += 32;
return colen;
}
/*===========================================================================*/
static u16 addcustomoption(u8 *pospt)
{
u16 colen;
option_header_t *optionhdr;
optionfield64_t *of;

optionhdr = (option_header_t*)pospt;
optionhdr->option_code = SHB_CUSTOM_OPT;
colen = OH_SIZE;
memcpy(pospt +colen, &hcxmagic, 4);
colen += 4;
memcpy(pospt +colen, &hcxmagic, 32);
colen += 32;

colen += addoption(pospt +colen, OPTIONCODE_MACAP, 6, (char*)valakt.macap);
of = (optionfield64_t*)(pospt +colen);
of->option_code = OPTIONCODE_RC;
of->option_length = 8;
of->option_value = valakt.replaycount;
colen += 12;
colen += addoption(pospt +colen, OPTIONCODE_ANONCE, 32, (char*)valakt.anonce);
colen += addoption(pospt +colen, OPTIONCODE_MACCLIENT, 6, (char*)valakt.macclient);
colen += addoption(pospt +colen, OPTIONCODE_SNONCE, 32, (char*)valakt.snonce);
colen += addoption(pospt +colen, OPTIONCODE_WEAKCANDIDATE, strnlen(valakt.weakcandidate, PSK_MAX), valakt.weakcandidate);
colen += addoption(pospt +colen, 0, 0, NULL);
optionhdr->option_length = colen -OH_SIZE;
return colen;
}
/*===========================================================================*/
static inline void writeepb()
{
ssize_t epblen;
u16 padding;
total_length_t *totallength;

epbhdr = (enhanced_packet_block_t*)epb;
epblen = EPB_SIZE;
epbhdr->block_type = EPBID;
epbhdr->interface_id = 0;
epbhdr->cap_len = packetlen;
epbhdr->org_len = packetlen;
epbhdr->timestamp_high = tsakt >> 32;
epbhdr->timestamp_low = (u32)tsakt &0xffffffff;
padding = (4 -(epbhdr->cap_len %4)) %4;
epblen += packetlen;
memset(&epb[epblen], 0, padding);
epblen += padding;
epblen += addoption(epb +epblen, SHB_EOC, 0, NULL);
totallength = (total_length_t*)(epb +epblen);
epblen += TOTAL_SIZE;
epbhdr->total_length = epblen;
totallength->total_length = epblen;
if(write(fd_pcapng, &epb, epblen) != epblen) errorcount++;
return;	
}
/*---------------------------------------------------------------------------*/
static bool writeshb()
{
ssize_t shblen;
section_header_block_t *shbhdr;
total_length_t *totallength;
struct utsname unameData;
char sysinfo[256];
u8 shb[PCAPNG_BLOCK_SIZE];

memset(&shb, 0, PCAPNG_BLOCK_SIZE);
shblen = SHB_SIZE;
shbhdr = (section_header_block_t*)shb;
shbhdr->block_type = PCAPNGBLOCKTYPE;
shbhdr->byte_order_magic = PCAPNGMAGICNUMBER;
shbhdr->major_version = PCAPNG_MAJOR_VER;
shbhdr->minor_version = PCAPNG_MINOR_VER;
shbhdr->section_length = -1;
if(uname(&unameData) == 0)
	{
	shblen += addoption(shb +shblen, SHB_HARDWARE, strlen(unameData.machine), unameData.machine);
	sprintf(sysinfo, "%s %s", unameData.sysname, unameData.release);
	shblen += addoption(shb +shblen, SHB_OS, strlen(sysinfo), sysinfo);
	sprintf(sysinfo, "%s %s", VERSIONNAME, VERSIONTAG);
	shblen += addoption(shb +shblen, SHB_USER_APPL, strlen(sysinfo), sysinfo);
	}
shblen += addcustomoption(shb +shblen);
shblen += addoption(shb +shblen, SHB_EOC, 0, NULL);
totallength = (total_length_t*)(shb +shblen);
shblen += TOTAL_SIZE;
shbhdr->total_length = shblen;
totallength->total_length = shblen;
if(write(fd_pcapng, &shb, shblen) != shblen) return false;
return true;
}
/*---------------------------------------------------------------------------*/
static bool writeidb()
{
ssize_t idblen;
interface_description_block_t *idbhdr;
total_length_t *totallength;
char tr[1];
u8 idb[PCAPNG_BLOCK_SIZE];

memset(&idb, 0, PCAPNG_BLOCK_SIZE);
idblen = IDB_SIZE;
idbhdr = (interface_description_block_t*)idb;
idbhdr->block_type = IDBID;
idbhdr->linktype = DLT_IEEE802_11_RADIO;
idbhdr->reserved = 0;
idbhdr->snaplen = PCAPNG_SNAPLEN;
idblen += addoption(idb +idblen, IF_NAME, strnlen(ifakt->ifname, IFNAMSIZ), ifakt->ifname);
idblen += addoption(idb +idblen, IF_MACADDR, 6, (char*)ifakt->hwmac);
tr[0] = TSRESOL_USEC;
idblen += addoption(idb +idblen, IF_TSRESOL, 1, tr);
idblen += addoption(idb +idblen, SHB_EOC, 0, NULL);
totallength = (total_length_t*)(idb +idblen);
idblen += TOTAL_SIZE;
idbhdr->total_length = idblen;
totallength->total_length = idblen;
if(write(fd_pcapng, &idb, idblen) != idblen) return false;
return true;
}
/*---------------------------------------------------------------------------*/
bool writecb()
{
ssize_t cblen;
custom_block_t *cbhdr;
optionfield64_t *of;
total_length_t *totallength;
u8 cb[PCAPNG_BLOCK_SIZE];

memset(&cb, 0, PCAPNG_BLOCK_SIZE);
cbhdr = (custom_block_t*)cb;
cblen = CB_SIZE;
cbhdr->block_type = CBID;
cbhdr->total_length = CB_SIZE;
memcpy(cbhdr->pen, &hcxmagic, 4);
memcpy(cbhdr->hcxm, &hcxmagic, 32);
cblen += addoption(cb +cblen, OPTIONCODE_MACAP, 6, (char*)valakt.macap);
of = (optionfield64_t*)(cb +cblen);
of->option_code = OPTIONCODE_RC;
of->option_length = 8;
of->option_value = valakt.replaycount;
cblen += 12;
cblen += addoption(cb +cblen, OPTIONCODE_ANONCE, 32, (char*)valakt.anonce);
cblen += addoption(cb +cblen, OPTIONCODE_MACCLIENT, 6, (char*)valakt.macclient);
cblen += addoption(cb +cblen, OPTIONCODE_SNONCE, 32, (char*)valakt.snonce);
cblen += addoption(cb +cblen, OPTIONCODE_WEAKCANDIDATE, strnlen(valakt.weakcandidate, PSK_MAX), valakt.weakcandidate);
cblen += addoption(cb +cblen, 0, 0, NULL);
totallength = (total_length_t*)(cb +cblen);
cblen += TOTAL_SIZE;
cbhdr->total_length = cblen;
totallength->total_length = cblen;
if(write(fd_pcapng, &cb, cblen) != cblen) return false;
return true;
}
/*---------------------------------------------------------------------------*/
static bool open_pcapng()
{
int c;
struct stat statinfo;
char timestring[16];
char pcapngname[PATH_MAX];

c = 0;
strftime(timestring, PATH_MAX, "%Y%m%d%H%M%S", localtime(&tvakt.tv_sec));
snprintf(pcapngname, PATH_MAX, "%s-%s.pcapng", timestring, ifakt->ifname);
while(stat(pcapngname, &statinfo) == 0)
	{
	snprintf(pcapngname, PATH_MAX, "%s-%s-%02d.pcapng", timestring, ifakt->ifname, c);
	c++;
	}
umask(0);
if((fd_pcapng = open(pcapngname, O_WRONLY | O_CREAT, 0777)) < 0) return false;
if(writeshb() == false) return false;
if(writeidb() == false) return false;
if(writecb() == false) return false;
return true;
}
/*===========================================================================*/
