/*===========================================================================*/
static void close_sockets()
{
if(bpf.filter != NULL)
	{
	if(fd_socket_rx > 0) setsockopt(fd_socket_rx, SOL_SOCKET, SO_DETACH_FILTER, &bpf, sizeof(bpf));
	free(bpf.filter);
	}
if(fd_socket_rx != 0) close(fd_socket_rx);
if(fd_socket_tx != 0) close(fd_socket_tx);
if(fd_socket_ioctl != 0) close(fd_socket_ioctl);
if(fd_socket_nl != 0) close(fd_socket_nl);
if(fd_socket_rt != 0) close(fd_socket_rt);
if(fd_gpsd != 0) close(fd_gpsd);
return;
}
/*===========================================================================*/
static bool open_socket_tx()
{
static struct sockaddr_ll saddr;

if((fd_socket_tx = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) return false;
memset(&saddr, 0, sizeof(saddr));
saddr.sll_family = PF_PACKET;
saddr.sll_ifindex = ifakt->ifindex;
saddr.sll_protocol = htons(ETH_P_ALL);
saddr.sll_halen = ETH_ALEN;
saddr.sll_pkttype = PACKET_OTHERHOST;
if(bind(fd_socket_tx, (struct sockaddr*) &saddr, sizeof(saddr)) < 0) return false;
return true;
}
/*===========================================================================*/
static bool open_socket_rx()
{
static struct sockaddr_ll saddr;
static struct packet_mreq mrq;
static int enable = 1;

if((fd_socket_rx = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) return false;

memset(&mrq, 0, sizeof(mrq));
mrq.mr_ifindex = ifakt->ifindex;
mrq.mr_type = PACKET_MR_PROMISC;
if(setsockopt(fd_socket_rx, SOL_PACKET, PACKET_ADD_MEMBERSHIP, &mrq, sizeof(mrq)) < 0) return false;

#if(LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
if(setsockopt(fd_socket_rx, SOL_PACKET, PACKET_IGNORE_OUTGOING, &enable, sizeof(int)) < 0) return false;
#endif

if(bpf.len > 0)
	{
	if(setsockopt(fd_socket_rx, SOL_SOCKET, SO_ATTACH_FILTER, &bpf, sizeof(bpf)) < 0) return false;
	}

memset(&saddr, 0, sizeof(saddr));
saddr.sll_family = PF_PACKET;
saddr.sll_ifindex = ifakt->ifindex;
saddr.sll_protocol = htons(ETH_P_ALL);
saddr.sll_halen = ETH_ALEN;
saddr.sll_pkttype = PACKET_OTHERHOST;
if(bind(fd_socket_rx, (struct sockaddr*) &saddr, sizeof(saddr)) < 0) return false;
while(wantstopflag == false)
	{
	packetlen = recv(fd_socket_rx, epb, PCAPNG_SNAPLEN, MSG_DONTWAIT);
	if(packetlen == -1) break;
	}
return true;
}
/*===========================================================================*/
static bool open_socket_nl()
{
struct sockaddr_nl saddr;
int nltxbuffsize = NLMSG_TXB_SIZE;
int nlrxbuffsize = NLMSG_RXB_SIZE;

if((fd_socket_nl = socket(AF_NETLINK, SOCK_RAW | SOCK_CLOEXEC, NETLINK_GENERIC)) < 0) return false;
if(setsockopt(fd_socket_nl, SOL_SOCKET, SO_SNDBUF, &nltxbuffsize, sizeof(nltxbuffsize)) < 0) return false;
if(setsockopt(fd_socket_nl, SOL_SOCKET, SO_RCVBUF, &nlrxbuffsize, sizeof(nlrxbuffsize)) < 0) return false;
if(fcntl(fd_socket_nl, F_SETFL, O_NONBLOCK) < 0) return false;
memset(&saddr, 0, sizeof(saddr));
saddr.nl_family = AF_NETLINK;
saddr.nl_pid = hcxpid;
if(bind(fd_socket_nl, (struct sockaddr *)&saddr, sizeof(saddr)) < 0) return false;
return true;
}
/*===========================================================================*/
static bool open_socket_rt()
{
struct sockaddr_nl saddr;
int nltxbuffsize = NLMSG_TXB_SIZE;
int nlrxbuffsize = NLMSG_RXB_SIZE;

if((fd_socket_rt = socket(AF_NETLINK, SOCK_RAW|SOCK_CLOEXEC, NETLINK_ROUTE)) < 0) return false;
if(setsockopt(fd_socket_rt, SOL_SOCKET, SO_SNDBUF, &nltxbuffsize, sizeof(nltxbuffsize)) < 0) return false;
if(setsockopt(fd_socket_rt, SOL_SOCKET, SO_RCVBUF, &nlrxbuffsize, sizeof(nlrxbuffsize)) < 0) return false;
memset(&saddr, 0, sizeof(saddr));
saddr.nl_family = AF_NETLINK;
saddr.nl_pid = getpid();
if(bind(fd_socket_rt, (struct sockaddr *)&saddr, sizeof(saddr)) < 0) return false;
return true;
}
/*===========================================================================*/
static bool open_socket_ioctl()
{
if((fd_socket_ioctl = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0) return false;
return true;
}
/*===========================================================================*/
static bool open_sockets_ctrl()
{
if(open_socket_nl() == false) return false;
if(open_socket_rt() == false) return false;
if(open_socket_ioctl() == false) return false;
return true;
}
/*===========================================================================*/
static bool open_sockets_data()
{
if(open_socket_rx() == false) return false;
if(open_socket_tx() == false) return false;
return true;
}
/*===========================================================================*/
static bool open_socket_gps()
{
int fcntlflags;
struct sockaddr_in gpsd_addr;
const char *gpsd_enable_nmea = "?WATCH={\"enable\":true,\"json\":false,\"nmea\":true}";

if((fd_gpsd = socket(AF_INET, SOCK_STREAM, 0)) < 0) return false;
memset(&gpsd_addr, 0, sizeof(struct sockaddr_in));
gpsd_addr.sin_family = AF_INET;
gpsd_addr.sin_port = htons(2947);
gpsd_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
if(connect(fd_gpsd, (struct sockaddr*) &gpsd_addr, sizeof(gpsd_addr)) < 0) return false;
if(write(fd_gpsd, gpsd_enable_nmea, 47) != 47) return false;
if((fcntlflags = fcntl(fd_gpsd, F_GETFL, 0)) < 0) return false;
if((fcntl(fd_gpsd, F_SETFL, fcntlflags | O_NONBLOCK)) < 0) return false;
return true;
}
/*===========================================================================*/
