#define _GNU_SOURCE
#include <arpa/inet.h>
#include <asm/types.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <ftw.h>
#include <getopt.h>
#include <ifaddrs.h>
#include <inttypes.h>
#include <libgen.h>
#include <linux/ethtool.h>
#include <linux/filter.h>
#include <linux/genetlink.h>
#include <linux/if_packet.h>
#include <linux/netlink.h>
#include <linux/nl80211.h>
#include <linux/rtnetlink.h>
#include <linux/sockios.h>
#include <linux/version.h>
#include <net/if.h>
#include <signal.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/epoll.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/timerfd.h>
#include <sys/types.h>
#include <sys/utsname.h>
#include <time.h>
#include <unistd.h>
#include <utime.h>

#include "include/hcx_types.h"
#include "include/hcx_radiotap.h"
#include "include/hcx_hcxlabtool.h"
#include "include/hcx_netlink-lite.h"
#include "include/hcx_wireless-lite.h"
#include "include/hcx_ieee80211.h"
#include "include/hcx_rpi.h"
#include "include/hcx_pcapng.h"
#include "functions/hcx_util.c"
#include "functions/hcx_file.c"
#include "functions/hcx_ioctl.c"
#include "functions/hcx_netlink.c"
#include "functions/hcx_sockets.c"
#include "functions/hcx_interfaces.c"
#include "functions/hcx_status.c"
#include "functions/hcx_rpi.c"
#include "functions/hcx_pcapng.c"
#include "functions/hcx_ieee80211tx.c"
#include "functions/hcx_ieee80211rx.c"
#include "functions/hcx_gpsd.c"
#include "functions/hcx_scan.c"

/*===========================================================================*/
static void close_lists()
{
if(iflist != 0) free(iflist);

if(aplist != 0) free(aplist);
if(proberesponselist != 0) free(proberesponselist);
return;
}
/*===========================================================================*/
/*===========================================================================*/
static bool init_values(char *userfrequencylist, char *userprotocol)
{
size_t c;
const char *wep = "we";
const char *nlp = "nl";
const char *weakcandidate = "12345678";

gettimeofday(&tvakt, NULL);
tsakt = ((u64)tvakt.tv_sec * 1000000ULL) + tvakt.tv_usec;
seed += tvakt.tv_usec & 0x7fffffff;
srand(seed);
if(init_scanlist(userfrequencylist) == false) return false;
if(scanlist[0] == 0) return false;
if(scanlist[1] == 0) staytime = 0;

if(userprotocol == NULL)
	{
	if((ifakt->iftype & IFTYPEWE) == IFTYPEWE) aktprotocol = IFTYPEWE;
	else if((ifakt->iftype & IFTYPENL) == IFTYPENL) aktprotocol = IFTYPENL;
	}
else if(userprotocol != NULL)
	{
	if(strncmp(wep, userprotocol, 2) == 0) aktprotocol = IFTYPEWE;
	else if(strncmp(nlp, userprotocol, 2) == 0) aktprotocol = IFTYPENL;
	else return false;
	}

strncpy(valakt.weakcandidate, weakcandidate, PSK_MAX);
for(c = 0; c < 32; c++)
	{
	valakt.anonce[c] = rand() % 0xff;
	valakt.snonce[c] = rand() % 0xff;
	}
valakt.replaycount = (rand() % 0xfff) + 0xf000;

packetoutptr = epbown + EPB_SIZE;
memcpy(packetoutptr, &rttxhdrd, RTTXHDR_SIZE);
return true;
}
/*===========================================================================*/
static bool init_lists()
{
if((iflist = (ifinfo_t*)calloc(IFINFOLIST_MAX, IFINFOLIST_SIZE)) == NULL) return false;

if((aplist = (aplist_t*)calloc(APLIST_MAX, APLIST_SIZE)) == NULL) return false;
if((proberesponselist = (aplist_t*)calloc(APLIST_MAX, APLIST_SIZE)) == NULL) return false;

packetptr = &epb[EPB_SIZE];
rth = (rt_t*)packetptr;
return true;
}
/*===========================================================================*/
/*===========================================================================*/
__attribute__ ((noreturn))
static inline void version(char *eigenname)
{
fprintf(stdout, "%s %s (C) %s ZeroBeat\n", eigenname, VERSIONTAG, VERSIONYEAR);
exit(EXIT_SUCCESS);
}
/*---------------------------------------------------------------------------*/
__attribute__ ((noreturn))
static inline void usage(char *eigenname)
{
fprintf(stdout, "%s %s  (C) %s ZeroBeat\n"
	"usage  : %s <options>\n"
	"\n"
	"short options:\n"
	"-i <interface> : interface (%s set monitor mode)\n"
	"                  default: best suitable interface\n"
	"-c <digit>     : set frequency (2437,2462,5600,...) or channel (1a,2a,36b ...)\n"
	"                  digit > 1000 is treated as frequency\n"
	"                  digit < 1000 is treated as channel\n"
	"                  important notice: channel numbers not unique\n"
	"                  it is mandatory to add band information to the channel number (e.g. 12a)\n"
	"                   band a: NL80211_BAND_2GHZ\n"
	"                   band b: NL80211_BAND_5GHZ\n"
	"                   band c: NL80211_BAND_6GHZ\n"
	"                   band d: NL80211_BAND_60GHZ\n"
	"                   band e: NL80211_BAND_S1GHZ (902 MHz)\n" 
	"                  to disable frequency mamagement, set this option to a single frequency/channel\n" 
	"-t <seconds>   : minimum stay time on channel before hopping to the next channel\n"
	"                  default: 1 second\n" 
	"                  to disable frequency mamagement, set this option to 0\n" 
	"-I             : show WLAN interfaces and quit\n"
	"-C             : show enabled channels and quit\n"
	"-h             : show this help\n"
	"-v             : show version\n"
	"\n",
	eigenname, VERSIONTAG, VERSIONYEAR, eigenname, eigenname);
fprintf(stdout, "long options:\n"
	"--bpf=<file>                   : input kernel space Berkeley Packet Filter (BPF) code\n"
	"                                  steps to create a BPF (it only has to be done once):\n"
	"                                  $ %s -m <interface>\n"
	"                                  create BPF to protect MACs\n"
	"                                  $ tcpdump -i <interface> not wlan addr2 11:22:33:44:55:66 -ddd > protect.bpf\n"
	"                                  recommended to protect own devices\n"
	"                                  create BPF to attack a MAC\n"
	"                                  $ tcpdump -i <interface> wlan addr1 11:22:33:44:55:66 or wlan addr2 11:22:33:44:55:66 or wlan addr3 11:22:33:44:55:66 -ddd > attack.bpf\n"
	"                                  it is strongly recommended to allow all PROBEREQUEST frames (wlan_type mgt && wlan_subtype probe-req)\n"
	"                                  $ tcpdump -i <interface> wlan addr1 11:22:33:44:55:66 or wlan addr2 11:22:33:44:55:66 or wlan addr3 11:22:33:44:55:66 or wlan addr3 ff:ff:ff:ff:ff:ff -ddd > attack.bpf\n"
	"                                  see man pcap-filter for a list of all filter options\n"
	"                                  add BPF code: \n"
	"                                  $ %s -i <interface> --bpfc=attack.bpf ...\n"
	"--disable_beacon               : do not transmit BEACON frames\n"
	"--disable_deauthentication     : do not send deauthentication frames\n"
	"--gpio_button=<digit>          : Raspberry Pi GPIO pin number of button (2...27)\n"
	"                                  push GIPO button to terminate program\n"
	"                                  default: %d (GPIO not in use)\n"
	"--gpio_statusled=<digit>       : Raspberry Pi GPIO number of status LED (2...27)\n"
	"                                  default: %d (GPIO not in use)\n"
	"--tot=<digit>                  : enable timeout timer in minutes\n"
	"--onsigterm=<reboot|poweroff>  : action when the program has been terminated by ctrl+c or GPIO button (reboot, poweroff)\n"
	"                                  default:  just terminate\n"
	"                                  reboot:   reboot system\n"
	"                                  poweroff: power off system\n"
	"--ongtot=<reboot|poweroff>     : action when the program has been terminated (reboot, poweroff)\n"
	"                                  default:  just terminate\n"
	"                                  reboot:   reboot system\n"
	"                                  poweroff: power off system\n"
	"--onerror=<reboot|poweroff>    : action when the program has been terminated (reboot, poweroff)\n"
	"                                  default:  just terminate\n"
	"                                  reboot:   reboot system\n"
	"                                  poweroff: power off system\n"
	"--protocol=<protocol>          : override protocol detection\n"
	"                                  we = WIRELESS EXTENSIONS\n"
	"                                  nl = NETLINK\n"
	"--gpsd                         : use GPSD\n"
	"                                  protocol: NMEA 0183 ($GPGGA $GPRMC)\n"
	"--help                         : show this help\n"
	"--version                      : show version\n"
	"\n"
	"Notice:\n"
	"This is a penetration testing tool!\n"
	"It is made to detect vulnerabilities in your NETWORK mercilessly!\n" 
	"To store entire traffic, run <tshark -i <interface> -w allframes.pcapng> in parallel\n"
	"\n",
	eigenname, eigenname, gpiobutton, gpiostatusled);
exit(EXIT_SUCCESS);
}
/*---------------------------------------------------------------------------*/
__attribute__ ((noreturn))
static inline void usageerror(char *eigenname)
{
fprintf(stdout, "%s %s (C) %s by ZeroBeat\n"
	"usage: %s -h for help\n", eigenname, VERSIONTAG, VERSIONYEAR, eigenname);
exit(EXIT_FAILURE);
}
/*===========================================================================*/
int main(int argc, char *argv[])
{
int auswahl = -1;
int index = 0;
unsigned int ifindex = 0;
char *userfrequency = NULL;
u8 buttonexit = 0;
u8 errorexit = 0;
u8 totexit = 0;
bool activebeaconflag = true;
char *userprotocol = NULL;
char *interfacename = NULL;
char *bpfname = NULL;
bool showinterfaceflag = false;
bool showfrequenciesflag = false;
bool gpsdflag = false;
const char *rebootstring = "reboot";
const char *poweroffstring = "poweroff";
const char *short_options = "i:c:t:IChv";
const struct option long_options[] =
{
	{"bpf",				required_argument,	NULL,	HCX_BPF},
	{"gpio_button",			required_argument,	NULL,	HCX_GPIO_BUTTON},
	{"gpio_statusled",		required_argument,	NULL,	HCX_GPIO_STATUSLED},
	{"disable_beacon",		no_argument,		NULL,	HCX_DISABLE_BEACON},
	{"disable_deauthentication",	no_argument,		NULL,	HCX_DISABLE_DEAUTHENTICATION},
	{"tot",				required_argument,	NULL,	HCX_TOT},
	{"onsigterm",			required_argument,	NULL,	HCX_ON_SIGTERM},
	{"ontot",			required_argument,	NULL,	HCX_ON_TOT},
	{"onerror",			required_argument,	NULL,	HCX_ON_ERROR},
	{"protocol",			required_argument,	NULL,	HCX_PROTOCOL},
	{"gps",				no_argument,		NULL,	HCX_GPS},
	{"version",			no_argument,		NULL,	HCX_VERSION},
	{"help",			no_argument,		NULL,	HCX_HELP},
	{NULL,				0,			NULL,	0}
};
optind = 1;
optopt = 0;
while((auswahl = getopt_long(argc, argv, short_options, long_options, &index)) != -1)
	{
	switch (auswahl)
		{
		case HCX_BPF:
		bpfname = optarg;
		break;

		case HCX_FREQUENCY:
		userfrequency = optarg;
		break;

		case HCX_DISABLE_BEACON:
		activebeaconflag = false;
		break;

		case HCX_DISABLE_DEAUTHENTICATION:
		deauthenticationflag = false;
		break;


		case HCX_INTERFACE_NAME:
		interfacename = optarg;
		break;

		case HCX_SHOW_INTERFACES:
		showinterfaceflag = true;
		break;

		case HCX_SHOW_FREQUENCIES:
		showfrequenciesflag = true;
		break;

		case HCX_GPIO_BUTTON:
		gpiobutton = strtol(optarg, NULL, 10);
		if((gpiobutton < 2) || (gpiobutton > 27))
			{
			fprintf(stderr, "invalid GPIO option\n");
			exit(EXIT_FAILURE);
			}
		break;

		case HCX_GPIO_STATUSLED:
		gpiostatusled = strtol(optarg, NULL, 10);
		if((gpiostatusled < 2) || (gpiostatusled > 27))
			{
			fprintf(stderr, "invalid GPIO option\n");
			exit(EXIT_FAILURE);
			}
		break;

		case HCX_STAYTIME:
		staytime = strtoul(optarg, NULL, 10);
		break;

		case HCX_TOT:
		if((tottime = strtoul(optarg, NULL, 10)) < 1)
			{
			fprintf(stderr, "time out timer must be > 0\n");
			exit(EXIT_FAILURE);
			}
		tottime *= 60;
		break;

		case HCX_ON_SIGTERM:
		if(strncmp(rebootstring, optarg, 8) == 0) buttonexit = WANT_REBOOT;
		else if(strncmp(poweroffstring, optarg, 8) == 0) buttonexit = WANT_POWEROFF;
		break;

		case HCX_ON_TOT:
		if(strncmp(rebootstring, optarg, 8) == 0) totexit = WANT_REBOOT;
		else if(strncmp(poweroffstring, optarg, 8) == 0) totexit = WANT_POWEROFF;
		break;

		case HCX_ON_ERROR:
		if(strncmp(rebootstring, optarg, 8) == 0) errorexit = WANT_REBOOT;
		else if(strncmp(poweroffstring, optarg, 8) == 0) errorexit = WANT_POWEROFF;
		break;

		case HCX_PROTOCOL:
		userprotocol = optarg;
		break;

		case HCX_GPS:
		gpsdflag = true;
		break;

		case HCX_HELP:
		usage(basename(argv[0]));
		break;

		case HCX_VERSION:
		version(basename(argv[0]));
		break;

		case '?':
		usageerror(basename(argv[0]));
		break;

		default:
		usageerror(basename(argv[0]));
		}
	}
//if(argc <= 1) usageerror(basename(argv[0]));
setbuf(stdout, NULL);
hcxpid = getpid();
if(init_lists() == false)
	{
	errorcount++;
	fprintf(stderr, "failed to initialize lists\n");
	goto exit;
	}
if((gpiobutton + gpiostatusled) > 0)
	{
	if(gpiobutton == gpiostatusled)
		{
		errorcount++;
		fprintf(stderr, "GPIO pin ERROR (same value of GPIO button and GPIO status LED)\n");
		goto exit;
		}
	if(initrpi() == false)
		{
		errorcount++;
		fprintf(stderr, "failed to initialize Raspberry Pi GPIO\n");
		goto exit;
		}
	}
if(interfacename != NULL)
	{
	if((ifindex = if_nametoindex(interfacename)) == 0)
		{
		errorcount++;
		fprintf(stderr, "failed to get interface index\n");
		goto exit;
		}
	}
if(bpfname != NULL)
	{
	if(readbpf(bpfname) == false)
		{
		errorcount++;
		fprintf(stderr, "failed read BPF\n");
		goto exit;
		}
	}
if(open_sockets_ctrl() == false)
	{
	errorcount++;
	fprintf(stderr, "failed to open sockets\n");
	goto exit;
	}
if(getwlaninterfaces() == false)
	{
	errorcount++;
	fprintf(stderr, "no suitable interface detected\n");
	goto exit;
	}
if(ifc == 0)
	{
	errorcount++;
	fprintf(stderr, "no suitable interface detected\n");
	goto exit;
	}
if(gpsdflag == true)
	{
	printf("flag\n");
	if(open_socket_gps() == false)
		{
		errorcount++;
		fprintf(stderr, "no suitable interface detected\n");
		goto exit;
		}
	}
if(showinterfaceflag == true) showinterfaces();
if(showfrequenciesflag == true) showfrequencies();
if((showinterfaceflag == true) || (showfrequenciesflag == true)) goto exit;
if((ifakt = getinterface(ifindex)) == NULL)
	{
	errorcount++;
	fprintf(stderr, "no suitable interface detected\n");
	goto exit;
	}
if(getuid() != 0)
	{
	errorcount++;
	fprintf(stderr, "%s must be run as root\n", basename(argv[0]));
	goto exit;
	}
if(set_interface_down() == false)
	{
	errorcount++;
	fprintf(stderr, "failed to set interface down\n");
	goto exit;
	}
if(set_interface_up() == false)
	{
	errorcount++;
	fprintf(stderr, "failed to set interface up\n");
	goto exit;
	}
if(open_sockets_data() == false)
	{
	errorcount++;
	fprintf(stderr, "failed to open rx socket\n");
	goto exit;
	}
if(init_values(userfrequency, userprotocol) == false)
	{
	errorcount++;
	fprintf(stderr, "failed to calculate initial values\n");
	goto exit;
	}
if(open_pcapng() == false)
	{
	errorcount++;
	fprintf(stderr, "failed to to open dump file\n");
	goto exit;
	}
if(aktprotocol == IFTYPEWE) fprintf(stdout, "starting %s on %s (%s) using WIRELESS EXTENSIONS...\n", basename(argv[0]), ifakt->ifname, ifakt->driver);
else if(aktprotocol == IFTYPENL) fprintf(stdout, "starting %s on %s (%s) using NETLINK...\n", basename(argv[0]), ifakt->ifname, ifakt->driver);
else
	{
	errorcount++;
	fprintf(stderr, "protocol detection failed\n");
	goto exit;
	}
if(bpf.len <= 0) fprintf(stderr, "BPF is unset. Make sure hcxlabtool is running in a 100%% controlled environment!\n");

//check_injection
//rcascan

if(fd_scan(activebeaconflag) == false)
	{
	errorcount++;
	fprintf(stderr, "main loop failed\n");
	goto exit;
	}

exit:
if(fd_pcapng != 0) close(fd_pcapng);
close_sockets();
close_lists();
if(errorcount > 0) fprintf(stderr, "\n%zu errors during runtime\n", errorcount);

if(buttonflag == true)
	{
	if(buttonexit == WANT_REBOOT)
		{
		if(system("reboot") != 0) fprintf(stderr, "\ncan't reboot system\n");
		}
	else if(buttonexit == WANT_POWEROFF)
		{
		if(system("poweroff") != 0) fprintf(stderr, "\ncan't power off\n");
		}
	}
if(totflag == true) 
	{
	if(totexit == WANT_REBOOT)
		{
		if(system("reboot") != 0) fprintf(stderr, "\ncan't reboot system\n");
		}
	else if(totexit == WANT_POWEROFF)
		{
		if(system("poweroff") != 0) fprintf(stderr, "\ncan't power off\n");
		}
	}
if (errorcount > ERROR_MAX)
	{
	if(errorexit == WANT_REBOOT)
		{
		if(system("reboot") != 0) fprintf(stderr, "\ncan't reboot system\n");
		}
	else if(errorexit == WANT_POWEROFF)
		{
		if(system("poweroff") != 0) fprintf(stderr, "\ncan't power off\n");
		}
	}
fprintf(stdout, "\nbye-bye\n");
return EXIT_SUCCESS;
}
/*===========================================================================*/
