/*===========================================================================*/
#define VERSIONTAG		"2.0.0"
#define VERSIONYEAR		"2023"
#define VERSIONNAME		"hcxlabtool"


#define HCX_BPF				1
#define HCX_DISABLE_BEACON		2
#define HCX_DISABLE_DEAUTHENTICATION	3
#define HCX_GPIO_BUTTON			4
#define HCX_GPIO_STATUSLED		5
#define HCX_TOT				6
#define HCX_ON_SIGTERM			7
#define HCX_ON_TOT			8
#define HCX_ON_ERROR			9
#define HCX_PROTOCOL			10
#define HCX_GPS				11
#define HCX_INTERFACE_NAME	'i'
#define HCX_FREQUENCY		'c'
#define HCX_SHOW_INTERFACES	'I'
#define HCX_SHOW_FREQUENCIES	'C'
#define HCX_STAYTIME		't'

#define HCX_HELP		'h'
#define HCX_VERSION		'v'

#define PCAPNG_SNAPLEN		0xffff

#define SCANLIST_MAX		1012

#define WANT_EXIT		0b00000000
#define WANT_REBOOT		0b00000010
#define WANT_POWEROFF		0b00000100

#define ERROR_MAX		100

#define EPOLL_EVENTS_MAX	10
#define STAYTIME		2
#define STAYTIMEPLUS		200000ULL
#define FDMSECRX		-1
#define FDUSEC1			100000000UL /* 10 second GPIO timer */
#define FDSEC2			10
#define TSTIMER1		1000000ULL /* (= 1 sec) */

#define EAPOLM2TIMEOUT		100000ULL
#define EAPOLM3TIMEOUT		40000ULL

#define PSK_MAX			64
#define ESSID_MAX		32
#define NONCE_MAX		32
#define PMKID_MAX		16
#define APLIST_MAX		255

#define GPSDB_SIZE		0x1024

/*===========================================================================*/
/*===========================================================================*/
typedef struct __attribute__((__packed__))
{
u8	iftype;
#define	IFTYPENL	0b00000001
#define	IFTYPEWE	0b00000010
#define	IFTYPENLWE	0b00000011
#define	IFTYPEMON	0b00000100
u32	phyindex;
u32	ifindex;
u8	hwmac[6];
u8	vimac[6];
#define FREQMAX		255
size_t	cf;
u32	frequency[FREQMAX];
u16	channel[FREQMAX];
u32	txpwr[FREQMAX];
char	ifname[IFNAMSIZ];
#define ETHTOOL_STD_LEN	32
char	driver[ETHTOOL_STD_LEN];
char	version[ETHTOOL_STD_LEN];
char	fw_version[ETHTOOL_FWVERS_LEN];
char	bus_info[ETHTOOL_BUSINFO_LEN];
char	erom_version[ETHTOOL_EROMVERS_LEN];
}ifinfo_t;
#define IFINFOLIST_SIZE (sizeof(ifinfo_t))
#define IFINFOLIST_MAX 255
/*---------------------------------------------------------------------------*/
static int sort_iflist_by_phyname(const void *a, const void *b)
{
const ifinfo_t *ia = (const ifinfo_t *)a;
const ifinfo_t *ib = (const ifinfo_t *)b;

if(ia->phyindex > ib->phyindex) return 1;
else if(ia->phyindex < ib->phyindex) return -1;
if(ia->ifindex > ib->ifindex) return 1;
else if(ia->ifindex < ib->ifindex) return -1;
return 0;
}
/*---------------------------------------------------------------------------*/
static int sort_iflist_by_iftype(const void *a, const void *b)
{
const ifinfo_t *ia = (const ifinfo_t *)a;
const ifinfo_t *ib = (const ifinfo_t *)b;

if(ia->iftype < ib->iftype) return 1;
else if(ia->iftype > ib->iftype) return -1;
if(ia->cf < ib->cf) return 1;
else if(ia->cf > ib->cf) return -1;
if(ia->ifindex > ib->ifindex) return 1;
else if(ia->ifindex < ib->ifindex) return -1;
return 0;
}
/*===========================================================================*/
typedef struct __attribute__((__packed__)) 
{
 u64	replaycount;
 u8	anonce[NONCE_MAX];
 u8	snonce[NONCE_MAX];
 u8	macap[6];
 u8	macclient[6];
 char	weakcandidate[PSK_MAX];
}valakt_t;
/*===========================================================================*/
typedef struct __attribute__((__packed__)) 
{
u64	tsakt;
u64	tsinterval1;
u16	count1;
u16	channel;
u8	mac[6];
u8	macclient[6];
u8	status;
u8	essidlen;
u8	essid[ESSID_MAX];
u8	groupcipher;
u8	paircipher;
u8	akm;
#define AP_EAPOL_M1	0b00000001
#define AP_EAPOL_M2	0b00000010
#define AP_EAPOL_M3	0b00000100
#define AP_EAPOL_OK	0b00000111
#define AP_PMKID	0b00001000
}aplist_t;
#define APLIST_SIZE (sizeof(aplist_t))
/*---------------------------------------------------------------------------*/
static int sort_aplist_by_tsakt(const void *a, const void *b)
{
const aplist_t *ia = (const aplist_t *)a;
const aplist_t *ib = (const aplist_t *)b;

if(ia->tsakt < ib->tsakt) return 1;
else if(ia->tsakt > ib->tsakt) return -1;
return 0;
}
/*===========================================================================*/
typedef struct
{
u64	tsfirst;
u8	status;
u8	macap[6];
u8	kvm1;
u64	replaycountm1;
u8	noncem1[4];
u8	kvm2;
u64	replaycountm2;
}authseqakt_t;
#define AUTHSEQAKT_SIZE (sizeof(authseqakt_t))
/*===========================================================================*/
/* gloabl var */

static struct timeval tvakt;
static u64 tsakt;
static size_t ifc = 0;
static pid_t hcxpid = 0;
static int fd_socket_rt = 0;
static int fd_socket_nl = 0;
static int fd_socket_ioctl = 0;
static int fd_socket_rx = 0;
static int fd_socket_tx = 0;
static int fd_gpsd = 0;
static int fd_pcapng = 0;
static struct sock_fprog bpf;
static ifinfo_t *iflist = NULL, *ifakt = NULL;
static u8 aktprotocol = IFTYPEWE;
static size_t scanlistcount = 0;
static u32 scanlist[SCANLIST_MAX];
static u32 scanlistchannel[SCANLIST_MAX];

static bool wantstopflag = false;
static bool buttonflag = false;
static bool totflag = false;

static int gpiostatusled = 0;
static int gpiobutton = 0;

static unsigned int seed = 7;

static time_t tottime = 0;
static time_t staytime = STAYTIME;
static ssize_t gpsdlen = 0;
static size_t errorcount = 0;

static valakt_t valakt = { 0 };
static authseqakt_t authseqakt = { 0 };
static u8 gpsdb[GPSDB_SIZE] = { 0 };
/*===========================================================================*/
